@@ -1,12 +1,17 @@
+# TODO: CachedResponse may be better as a non-slotted subclass of ClientResponse.
+#     Will look into this when working on issue #67.
+import asyncio
 import json
 from datetime import datetime
 from http.cookies import SimpleCookie
 from logging import getLogger
 from typing import Any, Dict, Iterable, List, Mapping, Optional, Tuple, Union
+from unittest.mock import Mock
 
 import attr
 from aiohttp import ClientResponse, ClientResponseError, hdrs, multipart
 from aiohttp.client_reqrep import ContentDisposition, MappingProxyType, RequestInfo
+from aiohttp.streams import StreamReader
 from aiohttp.typedefs import RawHeaders, StrOrURL
 from multidict import CIMultiDict, CIMultiDictProxy, MultiDict, MultiDictProxy
 from yarl import URL
@@ -14,6 +19,7 @@ from yarl import URL
 # CachedResponse attributes to not copy directly from ClientResponse
 EXCLUDE_ATTRS = {
     '_body',
+    '_content',
     '_links',
     'created_at',
     'encoding',
@@ -52,11 +58,12 @@ class CachedResponse:
     status: int = attr.ib()
     url: URL = attr.ib(converter=URL)
     version: str = attr.ib()
-    _body: Any = attr.ib(default=None)
+    _body: Any = attr.ib(default=b'')
+    _content: StreamReader = attr.ib(default=None)
     _links: LinkItems = attr.ib(factory=list)
     cookies: SimpleCookie = attr.ib(default=None)
     created_at: datetime = attr.ib(factory=datetime.utcnow)
-    encoding: str = attr.ib(default=None)
+    encoding: str = attr.ib(default='utf-8')
     expires: Optional[datetime] = attr.ib(default=None)
     raw_headers: RawHeaders = attr.ib(factory=tuple)
     real_url: StrOrURL = attr.ib(default=None)
@@ -66,24 +73,29 @@ class CachedResponse:
     @classmethod
     async def from_client_response(cls, client_response: ClientResponse, expires: datetime = None):
         """Convert a ClientResponse into a CachedReponse"""
-        # Response may not have been read yet, if fetched by something other than CachedSession
-        if not client_response._released:
-            await client_response.read()
+        if isinstance(client_response, cls):
+            return client_response
 
         # Copy most attributes over as is
         copy_attrs = set(attr.fields_dict(cls).keys()) - EXCLUDE_ATTRS
         response = cls(**{k: getattr(client_response, k) for k in copy_attrs})
 
-        # Set some remaining attributes individually
+        # Read response content, and reset StreamReader on original response
+        if not client_response._released:
+            await client_response.read()
         response._body = client_response._body
-        response._links = [(k, _to_str_tuples(v)) for k, v in client_response.links.items()]
+        client_response.content = CachedStreamReader(client_response._body)
+
+        # Set remaining attributes individually
         response.expires = expires
+        response.links = client_response.links
         response.real_url = client_response.request_info.real_url
 
-        # The encoding may be unset even if the response has been read
+        # The encoding may be unset even if the response has been read, and
+        # get_encoding() does not handle certain edge cases like an empty response body
         try:
             response.encoding = client_response.get_encoding()
-        except RuntimeError:
+        except (RuntimeError, TypeError):
             pass
 
         if client_response.history:
@@ -92,6 +104,16 @@ class CachedResponse:
             )
         return response
 
+    @property
+    def content(self) -> StreamReader:
+        if self._content is None:
+            self._content = CachedStreamReader(self._body)
+        return self._content
+
+    @content.setter
+    def content(self, value: StreamReader):
+        self._content = value
+
     @property
     def content_disposition(self) -> Optional[ContentDisposition]:
         """Get Content-Disposition headers, if any"""
@@ -112,6 +134,7 @@ class CachedResponse:
         """Get headers as an immutable, case-insensitive multidict from raw headers"""
 
         def decode_header(header):
+            """Decode an individual (key, value) pair"""
             return (
                 header[0].decode('utf-8', 'surrogateescape'),
                 header[1].decode('utf-8', 'surrogateescape'),
@@ -126,22 +149,26 @@ class CachedResponse:
     @property
     def is_expired(self) -> bool:
         """Determine if this cached response is expired"""
-        return self.expires is not None and datetime.utcnow() > self.expires
+        # If for any reason the expiration check fails, consider it expired and fetch a new response
+        try:
+            return self.expires is not None and datetime.utcnow() > self.expires
+        except (AttributeError, TypeError, ValueError):
+            return False
 
     @property
-    def links(self) -> LinkMultiDict:
+    def links(self) -> MultiDictProxy:
         """Convert stored links into the format returned by :attr:`ClientResponse.links`"""
         items = [(k, _to_url_multidict(v)) for k, v in self._links]
         return MultiDictProxy(MultiDict([(k, MultiDictProxy(v)) for k, v in items]))
 
+    @links.setter
+    def links(self, value: Mapping):
+        self._links = [(k, _to_str_tuples(v)) for k, v in value.items()]
+
     @property
     def ok(self) -> bool:
         """Returns ``True`` if ``status`` is less than ``400``, ``False`` if not"""
-        try:
-            self.raise_for_status()
-            return True
-        except ClientResponseError:
-            return False
+        return self.status < 400
 
     @property
     def request_info(self) -> RequestInfo:
@@ -149,7 +176,7 @@ class CachedResponse:
             url=URL(self.url),
             method=self.method,
             headers=self.headers,
-            real_url=URL(self.real_url),
+            real_url=URL(str(self.real_url)),
         )
 
     def get_encoding(self):
@@ -165,8 +192,8 @@ class CachedResponse:
     def raise_for_status(self):
         if self.status >= 400:
             raise ClientResponseError(
-                self.request_info,  # type: ignore  # These types are interchangeable
-                tuple(),
+                self.request_info,
+                self.history,
                 status=self.status,
                 message=self.reason,
                 headers=self.headers,
@@ -176,13 +203,58 @@ class CachedResponse:
         """Read response payload."""
         return self._body
 
-    def release(self):
-        """No-op function for compatibility with ClientResponse"""
+    def reset(self):
+        """Reset the stream reader to re-read a streamed response"""
+        self._content = None
 
     async def text(self, encoding: Optional[str] = None, errors: str = "strict") -> str:
         """Read response payload and decode"""
         return self._body.decode(encoding or self.encoding, errors=errors)
 
+    # No-op/placeholder properties and methods that don't apply to a CachedResponse, but provide
+    # compatibility with aiohttp.ClientResponse
+    # ----------
+
+    @property
+    def _released(self):
+        return True
+
+    @property
+    def connection(self):
+        return None
+
+    async def __aenter__(self) -> 'CachedResponse':
+        return self
+
+    async def close(self):
+        pass
+
+    async def wait_for_close(self):
+        pass
+
+    def release(self):
+        pass
+
+    async def start(self):
+        pass
+
+    async def terminate(self):
+        pass
+
+
+class CachedStreamReader(StreamReader):
+    """A StreamReader loaded from previously consumed response content. This feeds cached data into
+    the stream so it can support all the same behavior as the original stream: async iteration,
+    chunked reads, etc.
+    """
+
+    def __init__(self, body: bytes = None):
+        body = body or b''
+        protocol = Mock(_reading_paused=False)
+        super().__init__(protocol, limit=len(body), loop=asyncio.get_event_loop())
+        self.feed_data(body)
+        self.feed_eof()
+
 
 AnyResponse = Union[ClientResponse, CachedResponse]
 
