@@ -38,7 +38,8 @@ async def test_session__init_posarg():
 @patch.object(ClientSession, '_request')
 async def test_session__cache_hit(mock_request):
     cache = MagicMock(spec=CacheBackend)
-    cache.request.return_value = AsyncMock(is_expired=False), CacheActions()
+    response = AsyncMock(is_expired=False, url=URL('https://test.com'))
+    cache.request.return_value = response, CacheActions()
     session = CachedSession(cache=cache)
 
     await session.get('http://test.url')
@@ -73,3 +74,23 @@ async def test_session__default_attrs(mock_request):
 
     response = await session.get('http://test.url')
     assert response.from_cache is False and response.is_expired is False
+
+
+@pytest.mark.parametrize(
+    "params",
+    [
+        {'param': 'value'},  # Dict of strings
+        {'param': 4.2},  # Dict of floats
+        (('param', 'value'),),  # Tuple of (key, value) pairs
+        'param',  # string
+    ],
+)
+@patch.object(ClientSession, '_request')
+async def test_all_param_types(mock_request, params) -> None:
+    """Ensure that CachedSession.request() acceepts all the same parameter types as aiohttp"""
+    cache = MagicMock(spec=CacheBackend)
+    cache.request.return_value = None, CacheActions()
+    session = CachedSession(cache=cache)
+
+    response = await session.get('http://test.url', params=params)
+    assert response.from_cache is False
